.data
# couleurs
red: .word 0x00ff0000
green: .word 0x0000ff00

# variables globales
unitWidth: .word 8
unitHeight: .word 8
dsplWidth: .word 256
dsplHeight: .word 256

initial: .word 0		 	# variable qui contient l'adresse de la premiere case memoire de l'image (redefini automatiquement en fonction de l'allocation)

.text

# ->>--SET-BITMAP-DISPLAY-------------------------
# ->>---------------------------------------------
#  Set Bitmap Display (doit être coherent avec les variables globales de .data) :
# 
#  Unit Width in Pixels     = 8
#  Unit Height in Pixels    = 8
#  Display Width in Pixels  = 256
#  Display Height in Pixels = 256
#  Base address for display = 0x10040000 (heap)
# -<<---------------------------------------------
# -<<---------------------------------------------


# ->>--REPRESENTION-DE-L-IMAGE--------------------
# ->>---------------------------------------------
#  Le choix de la representation des elements de l'image se fera par des adresses memoires
#  la representation en adresses etant plus facile a stocker et plus simple a manipuler dans le cardre d'un `snake game`
# -<<---------------------------------------------
# -<<---------------------------------------------


# ->>---------------------------------------------
# MAIN
_main_:
	jal _I_creer_ 		# creation de l'image
	li $a0, 8		# nombre d'obstacle
	jal _O_creer_ 		# creation des obstacles
	j _exit_
# -<<---------------------------------------------

# ->>---------------------------------------------
_I_largeur_:
# 	Donne la largeur en unité de pixel
# ARGS: VOID
# RETU:	$v0<largeur de l'image>
# PREC: NONE
# prologue	
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	lw $t0, dsplWidth	# variable resolution en LARGEUR de l'image 
	lw $t1, unitWidth	# variable unité de representation des pixels en LARGEUR de l'image
	div $v0, $t0, $t1
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4
	
	jr $ra
# -<<---------------------------------------------


# ->>---------------------------------------------
_I_hauteur_:
# 	Donne la hauteur en unité de pixel
# ARGS: VOID
# RETU:	$v0<hauteur de l'image>
# PREC: NONE
# prologue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	lw $t0, dsplHeight	# variable resolution en HAUTEUR de l'image
	lw $t1, unitHeight	# variable unité de representation des pixels en HAUTEUR de l'image
	div $v0, $t0, $t1
# epilgue
	lw $ra, ($sp)
	add $sp, $sp, 4
	
	jr $ra
# -<<---------------------------------------------


# ->>---------------------------------------------
_I_creer_:
# 	Alloue une plage memoires pour l'image
# ARGS: VOID
# RETU:	$v0<adresse du tableau crée>
# PREC: NONE
# prologue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	jal _I_largeur_
	move $t9, $v0
	jal _I_hauteur_
	move $t8, $v0
	mul $a0, $t9, $t8	# passage en argument de `taille` pour l'allocation
	mul $a0, $a0, 4 	# passage en octet de `taille`
	li $v0, 9		# srbk (allocation memoire)
	syscall
	
	# modification de la valeur de `initial`
	la $t0, initial		# set premiere adresse
	sw $v0, ($t0) 		# set premiere adresse
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4

	jr $ra
# -<<---------------------------------------------

# ->>---------------------------------------------
_I_coordToAddress_:
# 	Donne l'adresse en fonction des coordonnées (y,x)
# ARGS: $a0<y ordonnée>, $a1<x abscisse>
# RETU:	$v0<adresse>
# 
# PREC: y >= 0 && x >= 0
# porlogue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
#  formule : adresse = `initial` + (y-1)*largeur*4 + (x-1)*4
	sub $a0, $a0, 1		#  y-1
	jal _I_largeur_
	mul $a0, $a0, $v0	#  (y-1)*largeur
	mul $a0, $a0, 4		#  (y-1)*largeur*4
	
	sub $a1, $a1, 1		#  x-1
	mul $a1, $a1, 4		#  (x-1)*4
	
	lw $v0, initial
	
	add $v0, $v0, $a0
	add $v0, $v0, $a1
	
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4
# return
	jr $ra
# -<<---------------------------------------------

# ->>---------------------------------------------
_I_addressToCoord_:
# 	Donne les coordonnées (y,x) en fonction de l'adresse avec les configurations de l'image
# ARGS:	$a0<adresse>
# RETU: $v0<y ordonnée>, $v1<x abscisse>
# 
# PREC: $a0 >= initial<adresse de debut de l'image>

# porlogue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	lw $t0, initial		# chargement de l'adresse de debut de l'image
	sub $a0, $a0, $t0	# distance entre `initial` et $a0
	div $a0, $a0, 4 	# valeur en octet vers decimal
	addi $a0, $a0, 1 	# representation qui commence par 1 à `largeur`
	
	jal _I_largeur_
	move $v1, $v0
	li $v0, 1 		# affectation de 1 dans $v0, qui est `y`, ordonnee, a incrementer si modulo possible
	ble $a0, $v1, next 	# premier teste si modulo
mod:	sub $a0, $a0, $v1
	add $v0, $v0, 1
	bgt $a0, $v1, mod
next:	
	move $v1, $a0 		# stockage de x, abscisse, dans $v1
	
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4
# return
	jr $ra
# -<<---------------------------------------------

# ->>---------------------------------------------
_I_plot_:
# 	Affiche un pixel ($a0, $a1) de couleur $a2
# ARGS: $a0<y>, $a1<x>, $a2<couleur>
# RETU: VOID
# 
# PREC: y >= 0 && x >= 0 && couleur<0xXXXXXXXX>

# prologue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	jal _I_coordToAddress_
	sw $a2, ($v0)
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4
# return
	jr $ra
# -<<---------------------------------------------

	



# ->>---------------------------------------------
_O_creer_:
# 	Crée un tableau d'obstacle (bordure et obstacles au hazard)
# ARGS; $a0<nombre obstacles>
# RETU: $v0<adresse du tableau>
# PREC: NONE
# prologue
	sub $sp, $sp, 16
	sw $ra, ($sp)
	sw $s0, 4($sp)
	sw $s1, 8($sp)
	sw $s2, 12($sp)
# corps
	jal _I_largeur_
	add $t9, $v0, $v0
	
	jal _I_hauteur_
	add $t8, $v0, $v0
	
	add $t0, $t9, $t8
	sub $t0, $t0, 4 	# $t0 = nombre bordure  = (largeur*2 + hauteur*2) - 4
	move $s2, $t0		# sauvegarde du nombre de borure
	
	move $s0, $a0		# sauvegarde nombre d'obstacle à cause de `syscall`
	add $a0, $t0, $a0 	# plus les obstacles au hasard
	mul $a0, $a0, 4		# a0 taille du tableau d'obstacle EN OCTET
	li $v0, 9 		# allocation memoire
	syscall
	
	move $a0, $v0		# recuperation de l'adresse allouée
	
	move $s1, $v0		# sauvegarde de l'adresse allouée
	move $a1, $s0

	jal _O_remplir_
	
	move $a1, $s2 		# nombre de bordures
	add $a1, $a1, $s0	# plus les obstacles
	move $a0, $s1		# passage en argument de l'adresse du tableau
	jal _O_affiche_
	
# epilogue
	lw $ra, ($sp)
	lw $s0, 4($sp)
	lw $s1, 8($sp)
	lw $s1, 12($sp)
	add $sp, $sp, 16
	
	jr $ra
# -<<---------------------------------------------


# ->>---------------------------------------------
_O_remplir_:
# 	Rempli le tableau d'obstacle
# ARGS:$a0<adresse tableau>, $a1<nombre obstacle>
# RETU: VOID
# 
# PREC: NONE

# prologue
	sub $sp, $sp, 16
	sw $ra, ($sp)
	sw $s0, 4($sp)
	sw $s1, 8($sp)
	sw $s2, 12($sp)
# corps
	jal _I_largeur_
	move $s0, $v0
	
	jal _I_hauteur_
	move $s1, $v0
	
	# Remplissage avec des bordures du haut
	lw $t0, initial
	move $t9, $s0
	
while_1:
	sw $t0, ($a0)		# stockage de l'adresse dans le tableau
	sub $t9, $t9, 1
	add $t0, $t0, 4 	# element suivant du tableau
	add $a0, $a0, 4 	# emplacement suivant du tableau
	bgtz $t9, while_1	# tant que compteur > 0
	
	
	# Remplissage avec des bordures laterale droite
	move $t9, $s1 		# hauteur comme condition d'arret
	sub $t9, $t9, 2 	# on a deja stocker la premier ligne et la dernier sera faite a part
	move $t8, $s0 		# recuperation de la largeur pour faire une saut à la ligne suivante par ajout
	mul $t8, $t8, 4 	# distance de la case courante avec la case de même colonne de la ligne suivante EN OCTET
	sub $t8, $t8, 4		# distance de la case courante avec la derniere case de la même ligne
	
while_2:
	sw $t0, ($a0)
	add $t0, $t0, $t8
	add $a0, $a0, 4 	# emplacement suivant du tableau
	sw $t0, ($a0)
	add $t0, $t0, 4 	# passage a la ligne suivante
	add $a0, $a0, 4 	# emplacement suivant du tableau
	sub $t9, $t9, 1 	# decrementation du compteur
	bgtz $t9, while_2 	# tant que compteur > 0
	
	move $t9, $s0 		# recuperation de la largeur pour faire une saut à la ligne suivante par ajout
	
while_3:
	sw $t0, ($a0)
	sub $t9, $t9, 1
	add $t0, $t0, 4 	# element suivant du tableau
	add $a0, $a0, 4 	# emplacement suivant du tableau
	bgtz $t9, while_3	# tant que compteur > 0
	
	
	move $t9, $s0
	move $t8, $s1
	mul $s2, $t9, $t8	# nombre de pixel de l'image
aleatoire:
	move $t8, $a0		# sauvegarde de valeur $a0 (adresse suivante à remplir du tableau)
	move $t7, $a1		# sauvegarde de valeur $a1 (nombre d'obstacle)
	li $v0, 42
	li $a0, 1
	move $a1, $s2		# valeur maximum du nombre aleatoire
	syscall
	move $t9, $a0		# recuperation du nombre aleatoire
	move $a0, $t8		# reaffectation de valeur $a0
	move $a1, $t7		# reaffectation de valeur $a1
	
	mul $t9, $t9, 4 	# convertion en OCTET
	lw $t0, initial		# recuperation de l'adresse de la premiere case de l'image
	add $t0, $t0, $t9	# $t0 : adresse de l'obstacle
	sw $t0, ($a0)
	add $a0, $a0, 4 	# emplacement suivant du tableau
	sub $a1, $a1, 1
	bgtz $a1, aleatoire 	# nombre d'obstacle > 0
	
# epilogue
	lw $ra, ($sp)
	lw $s0, 4($sp)
	lw $s1, 8($sp)
	lw $s2, 12($sp)
	add $sp, $sp, 16
	
	jr $ra
# -<<---------------------------------------------

# ->>---------------------------------------------
_O_affiche_:
# 	Rempli le tableau d'obstacle
# ARGS: $a0<address tableau>, $a1<taille>
# RETU: VOID
# 
# PREC: taille >= 0
# prologue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
	lw $t0, red 		# set color here !!
while:	lw $t1, ($a0) 		# acces 1er element du tableau
	sw $t0, ($t1) 		# stock a l'adresse du premier element
	sub $a1, $a1, 1
	add $a0, $a0, 4
	bgtz $a1, while
# epilogue
	lw $ra, ($sp)
	add $sp, $sp, 4
	
	jr $ra
# -<<---------------------------------------------


# ->>---------------------------------------------
_O_contient_:
# 	Teste si un pixel appartient ou non a tableau
# ARGS: $a0<adresse tableau>, $a1<taille>, $a2<adresse d'un pixel>
# RETU: $v0<0 ou 1>
# 
# PREC: taille >= 0

# prologue
	sub $sp, $sp, 4
	sw $ra, ($sp)
# corps
while_0:	
	beqz $a1, no 		# size =? 0
	beq $a0, $a2, yes 	# appartient ?
	sub $a1, $a1, 1 	# taille - 1
	add $a0, $a0, 4 	# element suivant du tableau
	bgtz $a1, while_0	# taille >? 0
no:	li $v0, 0
	j next_0
yes:	li $v0, 1
	j next_0
next_0:
# epilogue
	lw $ra, ($sp)
	sub $sp, $sp, 4
	
	jr $ra
# -<<---------------------------------------------


# ->>----------------------------------------------
_sleep_:
# 	Met une pause de 500 ms au processeur
# ARGS: VOID
# RETU: VOID
# PREC: NONE

# prologue
	sub $sp, $sp, 8
	sw $a0, ($sp)
	sw $v0, 4($sp)
# corps
	li $a0, 500 		# set time (millisecondes)
	li $v0, 32 		# MARS's sleep function
	syscall
# epilogue
	lw $a0, ($sp)
	lw $v0, 4($sp)
	add $sp, $sp, 8

	jr $ra
# -<<---------------------------------------------
# ->>---------------------------------------------
_exit_:
# 	arret complet du programme
	li $v0, 10
	syscall
# -<<---------------------------------------------
